package com.albertoventurini.parsley.examples.markdown;

import com.albertoventurini.parsley.grammar.Grammar;
import com.albertoventurini.parsley.grammar.rules.Rule;

import static com.albertoventurini.parsley.grammar.rules.Rules.*;

public class MarkdownGrammar extends Grammar {

    private final char NEWLINE = '\n';

    private final char BOLD_DELIMITER = '*';

    private final char ITALIC_DELIMITER = '_';

    private final Rule h1 = sequence(
            character('#'),
            takeWhile(c -> c != NEWLINE).as("text")
    ).as("h1");

    private final Rule h2 = sequence(
            string("##"),
            takeWhile(c -> c != NEWLINE).as("text")
    ).as("h2");

    private final Rule h3 = sequence(
            string("###"),
            takeWhile(c -> c != NEWLINE).as("text")
    ).as("h3");

    private final Rule headers = sequence(oneOf(h3, h2, h1), zeroOrMore(character(NEWLINE)));

    private final Rule plainText = takeWhile(c ->
            c != NEWLINE
            && c != BOLD_DELIMITER
            && c != ITALIC_DELIMITER
    ).as("text");

    private final Rule bold = sequence(
            character(BOLD_DELIMITER),
            takeWhile(c -> c != NEWLINE && c != BOLD_DELIMITER).as("bold"),
            character(BOLD_DELIMITER)
    ).as("boldWrapper");

    private final Rule italic = sequence(
            character(ITALIC_DELIMITER),
            takeWhile(c -> c != NEWLINE && c != ITALIC_DELIMITER).as("italic"),
            character(ITALIC_DELIMITER)
    ).as("italicWrapper");

    private final Rule text = oneOf(
            bold,
            italic,
            plainText);

    private final Rule paragraph = sequence(
            oneOrMore(text).as("paragraph"),
            zeroOrMore(character(NEWLINE))
    ).as("paragraphWrapper");

    private final Rule listItem = sequence(
            oneOf(string("* "), string("- ")),
            takeWhile(c -> c != NEWLINE).as("listItem"),
            character(NEWLINE)
    ).as("listItemWrapper");

    private final Rule list = sequence(oneOrMore(listItem).as("list"), zeroOrMore(character(NEWLINE)));

    private final Rule element = oneOf(headers, list, paragraph);

    private final Rule document = zeroOrMore(element);

    @Override
    public Rule startRule() {
        return document;
    }

    @Override
    protected Rule commentRule() {
        return null;
    }

    @Override
    protected boolean whitespace(final char c) {
        return c != '\n' && Character.isWhitespace(c);
    }
}
