package com.albertoventurini.parsley.examples.markdown;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class MarkdownGrammarTest {

    private final MarkdownGrammar grammar = new MarkdownGrammar();

    @Test
    public void shouldParseH1() {
        final var parseResult = grammar.parse("# hello world");

        assertTrue(parseResult.isPresent());
        assertEquals("# hello world", parseResult.get().getText());
        assertTrue(parseResult.get().getFirstDescendantByName("h1").isPresent());
        assertEquals("# hello world", parseResult.get().getFirstDescendantByName("h1").get().getText());
        assertTrue(parseResult.get().getFirstDescendantByName("text").isPresent());
        assertEquals("hello world", parseResult.get().getFirstDescendantByName("text").get().getText());
    }

    @Test
    public void shouldParseH2() {
        final var parseResult = grammar.parse("## hello world");

        assertTrue(parseResult.isPresent());
        assertEquals("## hello world", parseResult.get().getText());
        assertTrue(parseResult.get().getFirstDescendantByName("h2").isPresent());
        assertEquals("## hello world", parseResult.get().getFirstDescendantByName("h2").get().getText());
        assertNotNull(parseResult.get().getFirstDescendantByName("text"));
        assertTrue(parseResult.get().getFirstDescendantByName("text").isPresent());
        assertEquals("hello world", parseResult.get().getFirstDescendantByName("text").get().getText());
    }

    @Test
    public void shouldParseParagraph() {
        final var parseResult = grammar.parse("hello world");

        assertTrue(parseResult.isPresent());
        assertEquals("hello world", parseResult.get().getText());
        assertTrue(parseResult.get().getFirstDescendantByName("paragraph").isPresent());
        assertEquals("hello world", parseResult.get().getFirstDescendantByName("paragraph").get().getText());
    }

    @Test
    public void shouldParseParagraphWithBoldText() {
        final var parseResult = grammar.parse("hello world   *bold text*  normal text");

        assertTrue(parseResult.isPresent());
        assertTrue(parseResult.get().getFirstDescendantByName("paragraph").isPresent());
        assertEquals(3, parseResult.get().getFirstDescendantByName("paragraph").get().getChildren().size());
        assertEquals("bold text", parseResult.get().getFirstDescendantByName("bold").get().getText());
    }

    @Test
    public void shouldParseList() {
        final var parseResult = grammar.parse("hello world\n" +
                "\n" +
                "* first item\n" +
                "* second item\n" +
                "");

        assertTrue(parseResult.isPresent());
        assertTrue(parseResult.get().getFirstDescendantByName("paragraph").isPresent());
        assertEquals("hello world", parseResult.get().getFirstDescendantByName("paragraph").get().getText());
        assertTrue(parseResult.get().getFirstDescendantByName("list").isPresent());
        assertEquals(2, parseResult.get().getFirstDescendantByName("list").get().getChildren().size());
        assertEquals("first item", parseResult.get().getFirstDescendantByName("list").get().getChildren().get(0).getFirstDescendantByName("listItem").get().getText());

    }

}
