package com.albertoventurini.parsley.grammar;

import com.albertoventurini.parsley.grammar.rules.Rule;

import java.util.Optional;

public abstract class Grammar {

    protected abstract Rule startRule();

    protected abstract Rule commentRule();

    /**
     * Default whitespace predicate for grammars.
     * @param c a character
     * @return true if c is whitespace, false otherwise
     */
    protected boolean whitespace(final char c) {
        return Character.isWhitespace(c);
    }

    public Optional<ParseTree> parse(final String text) {
        final var ctx = new GrammarContext(text, commentRule()) {
            @Override
            protected boolean isWhitespace(final char c) {
                return whitespace(c);
            }
        };
        return startRule().apply(ctx);
    }
}
